﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace GameOfLife.Commands
{
    /// <summary>
    /// A special command that is used to add in any pre-made object that is defined externally
    /// in a file.
    /// </summary>
    internal class PremadeCommand : Command
    {
        /// <summary>
        /// Stores the name of the object/command.
        /// </summary>
        private string name;

        /// <summary>
        /// Stores the cells that are used in the premade object.
        /// </summary>
        private bool[,] objectGrid;

        /// <summary>
        /// Returns the name of the command, which will be different depending on the name
        /// of the object that it represents.
        /// </summary>
        public string CommandText
        {
            get { return name; }
        }

        /// <summary>
        /// Creates a new PremadeCommand, loading in a specific file that contains the
        /// object that this should represent.
        /// </summary>
        /// <param name="filename"></param>
        public PremadeCommand(string filename)
        {
            string[] lines = File.ReadAllLines(filename);
            int rows = lines.Length;

            name = Path.GetFileNameWithoutExtension(filename);
            int maxLength = DetermineLongestLine(lines);

            objectGrid = new bool[rows, maxLength];

            for (int row = 0; row < lines.Length; row++)
            {
                for (int column = 0; column < maxLength; column++)
                {
                    if (column >= lines[row].Length)
                    {
                        objectGrid[row, column] = false;
                    }
                    else
                    {
                        objectGrid[row, column] = (lines[row][column] == 'X');
                    }
                }
            }
        }

        /// <summary>
        /// Goes through a collection of strings and figures out which one has
        /// the most characters in it. This length is the one that is returned.
        /// </summary>
        /// <param name="lines"></param>
        /// <returns></returns>
        private static int DetermineLongestLine(string[] lines)
        {
            int maxLength = 0;
            for(int index = 0; index < lines.Length; index++)
            {
                if(lines[index].Length > maxLength)
                {
                    maxLength = lines[index].Length;
                }
            }

            return maxLength;
        }

        /// <summary>
        /// Executes the command, adding in the premade object that this represents to the grid.
        /// </summary>
        /// <param name="grid"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public bool Execute(Grid grid, string[] parameters)
        {
            // If they don't give their parameters correctly, error out.
            if (parameters.Length != 3)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("Error executing '{0}' command.", name);
                Console.WriteLine("\n  Example: '{0} 3 7' will place the object at x=3, y=7.", name);
                Console.ForegroundColor = ConsoleColor.White;
                return false;
            }

            try
            {
                int column = Convert.ToInt32(parameters[1]);
                int row = Convert.ToInt32(parameters[2]);

                for (int objectRow = 0; objectRow < this.objectGrid.GetLength(0); objectRow++)
                {
                    for (int objectColumn = 0; objectColumn < this.objectGrid.GetLength(1); objectColumn++)
                    {
                        grid.Set(row + objectRow, column + objectColumn, objectGrid[objectRow, objectColumn]);
                    }
                }

                return true;
            }
            catch (FormatException)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("Could not parse parameter.");
                Console.ForegroundColor = ConsoleColor.White;
                return false;
            }
            catch (Exception e)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(e.Message);
                Console.ForegroundColor = ConsoleColor.White;
                return false;
            }
        }

        /// <summary>
        /// Returns help text for the command.
        /// </summary>
        public string HelpText
        {
            get
            {
                string text = "";
                text += name + ":\n";
                text += "    Places a " + name + " on the grid at a specific location.\n";
                text += "    Usage: " + name + " {x} {y}\n";
                text += "    Example: " + name + " 4 6\n";
                return text;
            }
        }
    }
}
