﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using GameOfLife.Commands;

namespace GameOfLife
{
    class Program
    {
        /// <summary>
        /// Stores the grid being used.
        /// </summary>
        private static Grid grid;

        /// <summary>
        /// Indicates whether the game is running or waiting for a command to be typed.
        /// </summary>
        private static bool running = true;

        /// <summary>
        /// Stores the list of commands that are currently known.
        /// </summary>
        private static List<Command> commands;

        /// <summary>
        /// Runs the game.
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            InitializeAllCommands();

            grid = new Grid();
            grid.Randomize(0.8);

            // Start the animation loop on a separate thread.
            Thread thread = new Thread(RunAnimationLoop);
            thread.Start();
            Thread.Sleep(300);

            // Loop until they exit the program.
            while (true)
            {
                // Wait until a key is pressed, and stop the animation. This
                // will prepare things for recieving and interpreting commands from the user.
                Console.ReadKey(true);
                running = false;

                Thread.Sleep(300);

                Console.ForegroundColor = ConsoleColor.White;
                Console.WriteLine("Beginning command mode. Type 'help' for help and a list of all available commands, 'start' to begin animating, or 'exit' to exit the program.");

                // Loop until they start the animation again.
                while (true)
                {
                    Console.Write(">>> ");
                    string allInput = Console.ReadLine();
                    string[] parameters = allInput.Split(' ');
                    string command = parameters[0];
                    bool success = false;

                    // Handle a few special commands in a special way.
                    if (command == "start") { break; }
                    if (command == "exit") { Environment.Exit(0); }
                    if (command == "help") { new HelpCommand() { Commands = commands }.Execute(grid, parameters); }

                    // Otherwise, handle normal commands in the normal way:
                    // by checking if the command supplied matches one of the commands available,
                    // and if so, handing the data off to that command for processing.
                    for (int index = 0; index < commands.Count; index++)
                    {
                        if (command == commands[index].CommandText)
                        {
                            success = commands[index].Execute(grid, parameters);
                            break;
                        }
                    }

                    // When a command has been executed, we want to update the grid to reflect
                    // any changes that may have occurred as a result.
                    if (success)
                    {
                        grid.Draw();
                    }
                }

                // Back to running again...
                running = true;
            }
        }

        /// <summary>
        /// Builds the list of commands available in the game.
        /// </summary>
        private static void InitializeAllCommands()
        {
            // Load some standard commands.
            commands = new List<Command>();
            commands.Add(new RandomizeCommand());
            commands.Add(new TurnCommand());
            commands.Add(new GridCommand());
            commands.Add(new WrapCommand());
            commands.Add(new ClearCommand());

            // Look in the Resources directory for files that represent new objects that you can add.
            // (Note: you can add your own files in the Resources directory as well, which will automatically
            // create new commands for them.
            string path = "./Resources/";
            string[] premadeObjects = Directory.GetFiles(path);
            foreach (string objectFile in premadeObjects)
            {
                commands.Add(new PremadeCommand(objectFile));
            }
        }

        /// <summary>
        /// Runs the animation. This happens on a separate thread.
        /// </summary>
        private static void RunAnimationLoop()
        {
            // Loop forever, until the user quits the game.
            while (true)
            {
                // You only need to update the grid and draw it if the game is running.
                if (running)
                {
                    grid.Update();
                    grid.Draw();
                }

                // Wait for 1/4 of a second.
                Thread.Sleep(250);
            }
        }
    }
}
